
package eu.dnetlib.dhp.oa.graph.dump.eosc;

import static eu.dnetlib.dhp.common.SparkSessionSupport.runWithSparkSession;

import java.io.Serializable;
import java.util.*;
import java.util.stream.Collectors;

import org.apache.commons.io.IOUtils;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.function.FilterFunction;
import org.apache.spark.api.java.function.MapFunction;
import org.apache.spark.api.java.function.MapGroupsFunction;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.SaveMode;
import org.apache.spark.sql.SparkSession;
import org.jetbrains.annotations.Nullable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spark_project.jetty.util.StringUtil;

import eu.dnetlib.dhp.application.ArgumentApplicationParser;
import eu.dnetlib.dhp.eosc.model.Affiliation;
import eu.dnetlib.dhp.eosc.model.Country;
import eu.dnetlib.dhp.eosc.model.OrganizationPid;
import eu.dnetlib.dhp.eosc.model.Result;
import eu.dnetlib.dhp.schema.common.ModelConstants;
import eu.dnetlib.dhp.schema.oaf.Organization;
import eu.dnetlib.dhp.schema.oaf.Relation;
import scala.Tuple2;

/**
 * @author miriam.baglioni
 * @Date 27/07/22
 */
public class ExtendEoscResultWithOrganizationStep2 implements Serializable {
	private static final Logger log = LoggerFactory.getLogger(ExtendEoscResultWithOrganizationStep2.class);

	private final static String UNKNOWN = "UNKNOWN";

	public static void main(String[] args) throws Exception {
		String jsonConfiguration = IOUtils
			.toString(
				ExtendEoscResultWithOrganizationStep2.class
					.getResourceAsStream(
						"/eu/dnetlib/dhp/oa/graph/dump/eosc_extend_result_with_organization_parameters.json"));

		final ArgumentApplicationParser parser = new ArgumentApplicationParser(jsonConfiguration);
		parser.parseArgument(args);

		Boolean isSparkSessionManaged = Optional
			.ofNullable(parser.get("isSparkSessionManaged"))
			.map(Boolean::valueOf)
			.orElse(Boolean.TRUE);
		log.info("isSparkSessionManaged: {}", isSparkSessionManaged);

		final String inputPath = parser.get("sourcePath");
		log.info("inputPath: {}", inputPath);

		final String workingPath = parser.get("workingPath");
		log.info("workingPath: {}", workingPath);

		final String outputPath = parser.get("outputPath");
		log.info("outputPath: {}", outputPath);

		final String resultType = parser.get("resultType");
		log.info("resultType: {}", resultType);

		SparkConf conf = new SparkConf();

		runWithSparkSession(
			conf,
			isSparkSessionManaged,
			spark -> {
				Utils.removeOutputDir(spark, workingPath + resultType + "extendedaffiliation");
				Utils.removeOutputDir(spark, workingPath + resultType + "organization");
				Utils.removeOutputDir(spark, workingPath + resultType + "resultOrganization");
				addOrganizations(spark, inputPath, workingPath, resultType);
				dumpOrganizationAndRelations(spark, inputPath, workingPath, resultType);
			});
	}

	private static void dumpOrganizationAndRelations(SparkSession spark, String inputPath, String workingPath,
		String resultType) {
		Dataset<Relation> relation = Utils
			.readPath(spark, inputPath + "/relation", Relation.class)
			.filter(
				(FilterFunction<Relation>) r -> !r.getDataInfo().getDeletedbyinference() &&
					r.getSubRelType().equalsIgnoreCase(ModelConstants.AFFILIATION));

		Dataset<Organization> organization = Utils
			.readPath(spark, inputPath + "/organization", Organization.class)
			.filter((FilterFunction<Organization>) o -> !o.getDataInfo().getDeletedbyinference());

		Dataset<Result> result = Utils.readPath(spark, workingPath + resultType, Result.class);

		// result -> organization takes the relation of type affiliation having the source in the results related to
		// EOSC
		Dataset<Relation> eoscRelation = result
			.joinWith(relation, result.col("id").equalTo(relation.col("source")))
			.map((MapFunction<Tuple2<Result, Relation>, Relation>) t2 -> t2._2(), Encoders.bean(Relation.class));

		log.info("Number of affiliation relation for " + resultType + " = " + eoscRelation.count());

		// from eoscRelation select the organization
		Dataset<String> organizationIds = eoscRelation
			.joinWith(organization, eoscRelation.col("target").equalTo(organization.col("id")))
			.map(
				(MapFunction<Tuple2<Relation, Organization>, String>) t2 -> t2._2().getId(),
				Encoders.STRING())
			.distinct();

		organizationIds
			.joinWith(organization, organizationIds.col("value").equalTo(organization.col("id")))
			.map(
				(MapFunction<Tuple2<String, Organization>, eu.dnetlib.dhp.eosc.model.Organization>) t2 -> mapOrganization(
					t2._2()),
				Encoders.bean(eu.dnetlib.dhp.eosc.model.Organization.class))

			.filter(Objects::nonNull)

			.write()
			.mode(SaveMode.Overwrite)
			.option("compression", "gzip")
			.json(workingPath + resultType + "organization");

		eoscRelation
			.joinWith(organization, eoscRelation.col("target").equalTo(organization.col("id")))
			.map(
				(MapFunction<Tuple2<Relation, Organization>, eu.dnetlib.dhp.eosc.model.Relation>) t2 -> {
					if (isToBeRemovedOrg(t2._2()))
						return new eu.dnetlib.dhp.eosc.model.Relation();
					return eu.dnetlib.dhp.eosc.model.Relation
						.newInstance(t2._1().getSource(), t2._1().getTarget());

				},
				Encoders.bean(eu.dnetlib.dhp.eosc.model.Relation.class))
			.filter((FilterFunction<eu.dnetlib.dhp.eosc.model.Relation>) r -> StringUtil.isNotBlank(r.getSource()))
			.write()
			.mode(SaveMode.Overwrite)
			.option("compression", "gzip")
			.json(workingPath + resultType + "resultOrganization");

	}

	private static void addOrganizations(SparkSession spark, String inputPath, String workingPath,
		String resultType) {

		Dataset<Result> results = Utils
			.readPath(spark, workingPath + resultType, Result.class);

		Dataset<Relation> relations = Utils
			.readPath(spark, inputPath + "/relation", Relation.class)
			.filter(
				(FilterFunction<Relation>) r -> !r.getDataInfo().getDeletedbyinference() &&
					!r.getDataInfo().getInvisible() && r.getSubRelType().equalsIgnoreCase(ModelConstants.AFFILIATION));

		Dataset<Organization> organizations = Utils
			.readPath(spark, inputPath + "/organization", Organization.class)
			.filter((FilterFunction<Organization>) o -> !o.getDataInfo().getDeletedbyinference());

		Dataset<ResultOrganizations> resultOrganization = relations
			.joinWith(organizations, relations.col("source").equalTo(organizations.col("id")))
			.map((MapFunction<Tuple2<Relation, Organization>, ResultOrganizations>) t2 -> {
				ResultOrganizations rOrg = new ResultOrganizations();

				if (t2._2() != null) {

					rOrg.setResultId(t2._1().getTarget());
					Affiliation org = new Affiliation();
					org.setId(t2._2().getId());
					if (Optional.ofNullable(t2._2().getLegalname()).isPresent()) {
						org.setName(t2._2().getLegalname().getValue());
					} else {
						org.setName("");
					}
					HashMap<String, Set<String>> organizationPids = new HashMap<>();
					if (Optional.ofNullable(t2._2().getPid()).isPresent())
						t2._2().getPid().forEach(p -> {
							if (!organizationPids.containsKey(p.getQualifier().getClassid()))
								organizationPids.put(p.getQualifier().getClassid(), new HashSet<>());
							organizationPids.get(p.getQualifier().getClassid()).add(p.getValue());
						});
					List<OrganizationPid> pids = new ArrayList<>();
					for (String key : organizationPids.keySet()) {
						for (String value : organizationPids.get(key)) {
							OrganizationPid pid = new OrganizationPid();
							pid.setValue(value);
							pid.setType(key);
							pids.add(pid);
						}
					}
					org.setPid(pids);
					rOrg.setAffiliation(org);
					return rOrg;
				}
				return rOrg;

			}, Encoders.bean(ResultOrganizations.class))
			.filter((FilterFunction<ResultOrganizations>) ro -> ro.getResultId() != null);

		// resultOrganization.count();

		results
			.joinWith(resultOrganization, results.col("id").equalTo(resultOrganization.col("resultId")), "left")
			.groupByKey(
				(MapFunction<Tuple2<Result, ResultOrganizations>, String>) t2 -> t2._1().getId(), Encoders.STRING())
			.mapGroups(
				(MapGroupsFunction<String, Tuple2<Result, ResultOrganizations>, Result>) (s, it) -> addAffiliation(it),
				Encoders.bean(Result.class))
			.write()
			.mode(SaveMode.Overwrite)
			.option("compression", "gzip")
			.json(workingPath + resultType + "extendedaffiliation");

	}

	@Nullable
	private static ResultOrganizations getResultOrganizations(Tuple2<Relation, Organization> t2) {
		if (t2._2() != null) {
			Organization organization = t2._2();
			ResultOrganizations rOrg = new ResultOrganizations();
			rOrg.setResultId(t2._1().getTarget());
			Affiliation org = new Affiliation();
			org.setId(organization.getId());
			if (Optional.ofNullable(organization.getLegalname()).isPresent()) {
				org.setName(organization.getLegalname().getValue());
			} else {
				org.setName("");
			}
			HashMap<String, Set<String>> organizationPids = new HashMap<>();
			if (Optional.ofNullable(organization.getPid()).isPresent())
				organization.getPid().forEach(p -> {
					if (!organizationPids.containsKey(p.getQualifier().getClassid()))
						organizationPids.put(p.getQualifier().getClassid(), new HashSet<>());
					organizationPids.get(p.getQualifier().getClassid()).add(p.getValue());
				});
			List<OrganizationPid> pids = new ArrayList<>();
			for (String key : organizationPids.keySet()) {
				for (String value : organizationPids.get(key)) {
					OrganizationPid pid = new OrganizationPid();
					pid.setValue(value);
					pid.setType(key);
					pids.add(pid);
				}
			}
			org.setPid(pids);
			rOrg.setAffiliation(org);
			return rOrg;
		}
		return null;
	}

	private static Result addAffiliation(Iterator<Tuple2<Result, ResultOrganizations>> it) {
		Tuple2<Result, ResultOrganizations> first = it.next();
		if (first._2() == null) {
			return first._1();
		}
		Result ret = first._1();
		List<Affiliation> affiliation = new ArrayList<>();
		Set<String> alreadyInsertedAffiliations = new HashSet<>();
		affiliation.add(first._2().getAffiliation());
		alreadyInsertedAffiliations.add(first._2().getAffiliation().getId());
		it.forEachRemaining(res -> {
			if (!alreadyInsertedAffiliations.contains(res._2().getAffiliation().getId())) {
				affiliation.add(res._2().getAffiliation());
				alreadyInsertedAffiliations.add(res._2().getAffiliation().getId());
			}

		});
		ret.setAffiliation(affiliation);
		return ret;
	}

	private static eu.dnetlib.dhp.eosc.model.Organization mapOrganization(Organization org) {

		if (isToBeRemovedOrg(org))
			return null;

		eu.dnetlib.dhp.eosc.model.Organization organization = new eu.dnetlib.dhp.eosc.model.Organization();

		Optional
			.ofNullable(org.getLegalshortname())
			.ifPresent(value -> organization.setLegalshortname(value.getValue()));

		Optional
			.ofNullable(org.getLegalname())
			.ifPresent(value -> organization.setLegalname(value.getValue()));

		Optional
			.ofNullable(org.getWebsiteurl())
			.ifPresent(value -> organization.setWebsiteurl(value.getValue()));

		Optional
			.ofNullable(org.getAlternativeNames())
			.ifPresent(
				value -> organization
					.setAlternativenames(
						value
							.stream()
							.map(v -> v.getValue())
							.collect(Collectors.toList())));

		Optional
			.ofNullable(org.getCountry())
			.ifPresent(
				value -> {
					if (!value.getClassid().equals(UNKNOWN)) {
						organization
							.setCountry(
								Country.newInstance(value.getClassid(), value.getClassname()));
					}

				});

		Optional
			.ofNullable(org.getId())
			.ifPresent(value -> organization.setId(value));

		Optional
			.ofNullable(org.getPid())
			.ifPresent(
				value -> organization
					.setPid(
						value
							.stream()
							.map(p -> OrganizationPid.newInstance(p.getQualifier().getClassid(), p.getValue()))
							.collect(Collectors.toList())));

		return organization;
	}

	private static boolean isToBeRemovedOrg(Organization org) {
		if (Boolean.TRUE.equals(org.getDataInfo().getDeletedbyinference()))
			return true;
//		if (!Optional.ofNullable(org.getLegalname()).isPresent()
//			&& !Optional.ofNullable(org.getLegalshortname()).isPresent())
//			return true;
		return false;
	}

}
