
package eu.dnetlib.dhp.oa.graph.dump.eosc;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;

import org.apache.commons.io.FileUtils;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.sql.SparkSession;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.ObjectMapper;

import eu.dnetlib.dhp.eosc.model.Affiliation;
import eu.dnetlib.dhp.eosc.model.Organization;
import eu.dnetlib.dhp.eosc.model.Relation;
import eu.dnetlib.dhp.eosc.model.Result;

/**
 * @author miriam.baglioni
 * @Date 25/10/23
 */
public class ExtendAffiliationTest {

	private static final ObjectMapper OBJECT_MAPPER = new ObjectMapper();

	private static SparkSession spark;

	private static Path workingDir;

	private static final Logger log = LoggerFactory
		.getLogger(ExtendAffiliationTest.class);

	private static HashMap<String, String> map = new HashMap<>();

	@BeforeAll
	public static void beforeAll() throws IOException {
		workingDir = Files
			.createTempDirectory(ExtendAffiliationTest.class.getSimpleName());
		log.info("using work dir {}", workingDir);

		SparkConf conf = new SparkConf();
		conf.setAppName(ExtendAffiliationTest.class.getSimpleName());

		conf.setMaster("local[*]");
		conf.set("spark.driver.host", "localhost");
		conf.set("hive.metastore.local", "true");
		conf.set("spark.ui.enabled", "false");
		conf.set("spark.sql.warehouse.dir", workingDir.toString());
		conf.set("hive.metastore.warehouse.dir", workingDir.resolve("warehouse").toString());

		spark = SparkSession
			.builder()
			.appName(ExtendAffiliationTest.class.getSimpleName())
			.config(conf)
			.getOrCreate();
	}

	@AfterAll
	public static void afterAll() throws IOException {
		FileUtils.deleteDirectory(workingDir.toFile());
		spark.stop();
	}

	@Test
	public void ExtendEoscResultWithOrganizationTest() throws Exception {
		final String sourcePath = getClass()
			.getResource("/eu/dnetlib/dhp/oa/graph/dump/eosc/input")
			.getPath();

		final String workingPath = getClass()
			.getResource("/eu/dnetlib/dhp/oa/graph/dump/eosc/working/")
			.getPath();

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		spark
			.read()
			.textFile(workingPath + "publication")
			.write()
			.text(workingDir.toString() + "/working/publication");

		ExtendEoscResultWithOrganizationStep2.main(new String[] {
			"-isSparkSessionManaged", Boolean.FALSE.toString(),
			"-outputPath", workingDir.toString() + "/",
			"-sourcePath", sourcePath,
			"-resultType", "publication",
			"-workingPath", workingDir.toString() + "/working/"
		});

		/*
		 * affiliation relationships 20|13811704aa70::51a6ade52065e3b371d1ae822e07f1ff ->
		 * 50|06cdd3ff4700::93859bd27121c3ee7c6ee4bfb1790cba
		 */

		JavaRDD<Result> tmp = sc
			.textFile(workingDir.toString() + "/working/publicationextendedaffiliation")
			.map(item -> OBJECT_MAPPER.readValue(item, Result.class));

		Assertions.assertEquals(3, tmp.count());

		tmp.foreach(p -> System.out.println(OBJECT_MAPPER.writeValueAsString(p)));

		Assertions
			.assertEquals(
				1,
				tmp
					.filter(r -> Optional.ofNullable(r.getAffiliation()).isPresent() && r.getAffiliation().size() > 0)
					.count());

		Assertions
			.assertEquals(
				1,
				tmp
					.filter(r -> r.getId().equalsIgnoreCase("50|06cdd3ff4700::93859bd27121c3ee7c6ee4bfb1790cba"))
					.first()
					.getAffiliation()
					.size());

		List<Affiliation> affiliations = tmp
			.filter(r -> r.getId().equalsIgnoreCase("50|06cdd3ff4700::93859bd27121c3ee7c6ee4bfb1790cba"))
			.first()
			.getAffiliation();

		Assertions
			.assertTrue(
				affiliations.stream().anyMatch(a -> a.getName().equalsIgnoreCase("Doris Engineering (France)")));

		Affiliation organization = affiliations
			.stream()
			.filter(a -> a.getId().equalsIgnoreCase("20|13811704aa70::51a6ade52065e3b371d1ae822e07f1ff"))
			.findFirst()
			.get();
		Assertions.assertEquals("Doris Engineering (France)", organization.getName());
		Assertions
			.assertTrue(
				organization
					.getPid()
					.stream()
					.anyMatch(
						p -> p.getValue().equalsIgnoreCase("grid.432986.2") && p.getType().equalsIgnoreCase("grid")));
		Assertions
			.assertTrue(
				organization
					.getPid()
					.stream()
					.anyMatch(
						p -> p.getValue().equalsIgnoreCase("https://ror.org/03nd0ms94")
							&& p.getType().equalsIgnoreCase("ror")));
		Assertions.assertEquals(2, organization.getPid().size());

	}

	@Test
	public void selectEoscResults() throws Exception {

		final String sourcePath = getClass()
			.getResource("/eu/dnetlib/dhp/oa/graph/dump/eosc/input")
			.getPath();

		final String workingPath = getClass()
			.getResource("/eu/dnetlib/dhp/oa/graph/dump/eosc/working/")
			.getPath();

		ExtendEoscResultWithOrganizationStep2.main(new String[] {
			"-isSparkSessionManaged", Boolean.FALSE.toString(),
			"-outputPath", workingDir.toString() + "/",
			"-sourcePath", sourcePath,
			"-resultType", "publication",
			"-workingPath", workingPath
		});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<Organization> tmp = sc
			.textFile(workingDir.toString() + "/organization")
			.map(item -> OBJECT_MAPPER.readValue(item, Organization.class));

		JavaRDD<Relation> rels = sc
			.textFile(workingDir.toString() + "/resultOrganization")
			.map(item -> OBJECT_MAPPER.readValue(item, Relation.class));

		System.out.println(tmp.count());

		Assertions.assertEquals(2, tmp.count());

		Assertions.assertEquals(2, rels.count());

		rels.foreach(r -> Assertions.assertTrue(r.getSource().startsWith("50|")));
		rels.foreach(r -> Assertions.assertTrue(r.getTarget().startsWith("20|")));

	}

}
